'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var loom = require('@shopify/loom');
var babel = require('@rollup/plugin-babel');
var commonjs = require('@rollup/plugin-commonjs');
var nodeResolve = require('@rollup/plugin-node-resolve');
var externals = require('rollup-plugin-node-externals');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var babel__default = /*#__PURE__*/_interopDefaultLegacy(babel);
var commonjs__default = /*#__PURE__*/_interopDefaultLegacy(commonjs);
var nodeResolve__default = /*#__PURE__*/_interopDefaultLegacy(nodeResolve);
var externals__default = /*#__PURE__*/_interopDefaultLegacy(externals);

function rollupConfig(options) {
  return loom.createProjectBuildPlugin('BuildLibrary.RollupConfig', ({
    hooks,
    project
  }) => {
    // Define additional build variant to build esnext output
    hooks.targets.hook(targets => {
      return targets.map(target => {
        return options.esnext && target.default ? target.add({
          rollupEsnext: true
        }) : target;
      });
    }); // Define config for build variants

    hooks.target.hook(async ({
      target,
      hooks
    }) => {
      const isDefaultBuild = Object.keys(target.options).length === 0;
      const isEsnextBuild = Boolean(target.options.rollupEsnext);

      if (!(isDefaultBuild || isEsnextBuild)) {
        return;
      }

      hooks.configure.hook(async configuration => {
        var _configuration$rollup, _configuration$rollup2, _configuration$rollup3;

        (_configuration$rollup = configuration.rollupInput) === null || _configuration$rollup === void 0 ? void 0 : _configuration$rollup.hook(async input => {
          const absentInputEntries = await asyncFilter([...target.project.entries, ...target.project.binaries], ({
            root
          }) => project.fs.hasFile(project.fs.resolvePath(root)).then(hasFile => !hasFile));

          if (absentInputEntries.length) {
            const absentInputEntriesString = absentInputEntries.map(item => `"${item.root}"`).join(', ');
            throw new loom.DiagnosticError({
              title: `The following entry/binary root paths were defined in "${project.name}" but do not exist on disk: ${absentInputEntriesString}.`,
              suggestion: `Root paths in your pkg.entry() / pkg.binary() config must be valid file paths. Update any paths to match the filename on disk. e.g. use "pkg.entry({root: './src/index.EXTENSION_HERE'})" instead of "pkg.entry({root: './src/index'})"`
            });
          }

          const inputEntries = [...target.project.entries, ...target.project.binaries].map(({
            root
          }) => project.fs.resolvePath(root));

          if (inputEntries.length === 0) {
            throw new loom.DiagnosticError({
              title: `No inputs found for "${project.name}".`,
              suggestion: `Set a pkg.entry() in your loom.config that maps to a file on disk. Use 'pkg.entry({root: './src/index.js'})" to use the index.js file`
            });
          }

          return input.concat(inputEntries);
        });
        (_configuration$rollup2 = configuration.rollupPlugins) === null || _configuration$rollup2 === void 0 ? void 0 : _configuration$rollup2.hook(async plugins => {
          var _configuration$babelC;

          const babelConfig = await ((_configuration$babelC = configuration.babelConfig) === null || _configuration$babelC === void 0 ? void 0 : _configuration$babelC.run({}));
          const babelTargets = isEsnextBuild ? ['last 1 chrome versions'] : [options.targets];
          const packagePath = (await project.fs.hasFile('./package.json')) ? project.fs.resolvePath('./package.json') : [];
          return plugins.concat([externals__default["default"]({
            deps: true,
            packagePath
          }), nodeResolve__default["default"]({
            extensions: ['.js', '.jsx', '.ts', '.tsx']
          }), commonjs__default["default"](), babel__default["default"]({ ...babelConfig,
            // Options specific to @rollup/plugin-babel, these can not be
            // present on the `babelConfig` object
            extensions: ['.js', '.jsx', '.ts', '.tsx'],
            exclude: 'node_modules/**',
            babelHelpers: 'bundled',
            // Options that may be present on the `babelConfig` object but
            // we want to override
            envName: 'production',
            // @ts-expect-error targets is a valid babel option but @types/babel__core doesn't know that yet
            targets: babelTargets
          })]);
        });
        (_configuration$rollup3 = configuration.rollupOutputs) === null || _configuration$rollup3 === void 0 ? void 0 : _configuration$rollup3.hook(outputs => {
          const additionalOutputs = [];

          if (isDefaultBuild) {
            if (options.commonjs) {
              additionalOutputs.push({
                format: 'cjs',
                dir: project.fs.buildPath('cjs'),
                preserveModules: true,
                entryFileNames: '[name][assetExtname].js',
                exports: 'named'
              });
            }

            if (options.esmodules) {
              additionalOutputs.push({
                format: 'esm',
                dir: project.fs.buildPath('esm'),
                preserveModules: true,
                entryFileNames: '[name][assetExtname].mjs'
              });
            }
          } else if (isEsnextBuild) {
            if (options.esnext) {
              additionalOutputs.push({
                format: 'esm',
                dir: project.fs.buildPath('esnext'),
                preserveModules: true,
                entryFileNames: '[name][assetExtname].esnext'
              });
            }
          }

          return outputs.concat(additionalOutputs);
        });
      });
    });
  });
}

async function asyncFilter(arr, predicate) {
  return Promise.all(arr.map(predicate)).then(results => arr.filter((_v, index) => results[index]));
}

exports.rollupConfig = rollupConfig;
