'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var path = require('path');
var loom = require('@shopify/loom');
var findCommonAncestorPath = require('common-ancestor-path');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var findCommonAncestorPath__default = /*#__PURE__*/_interopDefaultLegacy(findCommonAncestorPath);

function writeEntrypoints(options) {
  return loom.createProjectBuildPlugin('BuildLibrary.Entrypoints', ({
    hooks,
    project,
    api
  }) => {
    hooks.steps.hook(steps => [...steps, createWriteEntrypointsStep({
      project,
      api
    }, options)]);
  });
}

function createWriteEntrypointsStep({
  project,
  api
}, options) {
  return api.createStep({
    id: 'BuildLibrary.Entrypoints',
    label: 'Write entrypoints'
  }, async () => {
    const entryConfigs = [];

    if (options.commonjs) {
      entryConfigs.push({
        exportStyle: 'commonjs',
        outputPath: project.fs.buildPath('cjs'),
        extension: '.js'
      });
    }

    if (options.esmodules) {
      entryConfigs.push({
        exportStyle: 'esmodules',
        outputPath: project.fs.buildPath('esm'),
        extension: '.mjs'
      });
    }

    if (options.esnext) {
      entryConfigs.push({
        exportStyle: 'esmodules',
        outputPath: project.fs.buildPath('esnext'),
        extension: '.esnext'
      });
    }

    await Promise.all(entryConfigs.map(config => writeEntries(project, config)));
  });
}

async function writeEntries(project, {
  extension,
  outputPath,
  exportStyle
}) {
  const commonEntryRoot = findCommonAncestorPath__default["default"](...(await Promise.all([...project.entries, ...project.binaries].map(async ({
    root
  }) => path.dirname(project.fs.resolvePath(root)))))); // This is present as a typeguard that should never trigger as
  // findCommonAncestorPath will only return null if you're on windows and you
  // feed it files that are on different drives - which should never occur if
  // you're building a project that is contained within a single folder

  if (commonEntryRoot === null) {
    throw new Error('Could not find a common ancestor folder for all your entrypoints');
  }

  await Promise.all(project.entries.map(async ({
    name,
    root
  }) => {
    const entryPath = project.fs.resolvePath(root);
    const entryRelativeToCommonRoot = path.relative(commonEntryRoot, entryPath);
    const pathToBuiltEntry = `./${path.relative(project.root, path.join(outputPath, `${entryRelativeToCommonRoot.replace(/\.[^.]+$/, extension)}`))}`;
    const quotedPathToBuiltEntry = JSON.stringify(pathToBuiltEntry);
    const rootEntryFile = `${name !== null && name !== void 0 ? name : 'index'}${extension}`;
    const entryPointContent = exportStyle === 'commonjs' ? cjsEntrypointContent(quotedPathToBuiltEntry) : esmEntrypointContent(quotedPathToBuiltEntry, await entryHasDefaultExport(entryPath, project));
    await project.fs.write(rootEntryFile, entryPointContent);
  }));
}

async function entryHasDefaultExport(entryPath, project) {
  try {
    const content = await project.fs.read(entryPath); // export default ...
    // export {Foo as default} from ...
    // export {default} from ...

    return /(?:export|as) default\b/.test(content) || /{default}/.test(content);
  } catch {
    return true;
  }
}

function cjsEntrypointContent(quotedPath) {
  return `module.exports = require(${quotedPath});`;
}

function esmEntrypointContent(quotedPath, hasDefault) {
  return [`export * from ${quotedPath};`, hasDefault ? `export {default} from ${quotedPath};` : false].filter(Boolean).join('\n');
}

exports.writeEntrypoints = writeEntrypoints;
